import datetime
from mimetypes import guess_type
import os
import time
from django.core.files import File
from forms.serializers import ProjectSerializer
from forms.models import Project
from notifications_app.views import send_push_notification
from notifications_app.models import Notification
from posts_app.utils import generate_video_thumbnail
from save_app.models import SavedPost
from social_app.models import Follow
from rest_framework import viewsets, status
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated
from django.shortcuts import get_object_or_404
from django.urls import reverse
from .models import Headshot, Post, Like, Comment, PostMedia, PostReport, ReportReason
from .serializers import HeadshotSerializer, PostReportSerializer, PostSerializer, LikeSerializer, CommentSerializer, ReportReasonSerializer
from userprofile_app.models import Profile
from django.contrib.auth import get_user_model
from rest_framework.views import APIView
from django.db.models import Q
from django.utils.timezone import now
from operator import itemgetter

User = get_user_model() 

class PostViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]

    def create(self, request):
        user = request.user
        data = request.data

        media_files = request.FILES.getlist('media')

        # print(f"Creating a post for user: {user}")
        # data['user'] = user.id
        # print("Request data:", data)
        print("Media files:", media_files)

        # Serialize and create the post
        post_serializer = PostSerializer(data=data, context={'request': request})
        if post_serializer.is_valid():
            try:
                post = post_serializer.save(user=user)  # Associate the user

                images = []
                video = None
                print(images)

                # for file in media_files:
                #     PostMedia.objects.create(post=post, media=file)
                for file in media_files:
                    print(f"Processing file: {file.name}")
                    print(f"File type: {type(file)}")
                    mime_type, _ = guess_type(file.name)
                    if mime_type and mime_type.startswith('image'):
                        images.append(file)
                        PostMedia.objects.create(post=post, media=file, media_type='image')
                    elif mime_type and mime_type.startswith('video'):
                        if video:  # Check if a video is already uploaded
                            return Response({
                                "status": "error",
                                "message": "Only one video is allowed per post."
                            }, status=status.HTTP_400_BAD_REQUEST)
                        video = file
                        # PostMedia.objects.create(post=post, media=video, media_type='video')

                # Check if the user is uploading both images and a video
                if images and video:
                    return Response({
                        "status": "error",
                        "message": "You cannot upload both images and a video in the same post."
                    }, status=status.HTTP_400_BAD_REQUEST)
                print(images[0])
                if images:
                    post.thumbnail.save(images[0].name, images[0], save=True)

                elif video:  # If a video is uploaded, generate a thumbnail
                    thumbnail_path = generate_video_thumbnail(video)
                    with open(thumbnail_path, 'rb') as thumb_file:
                        post.thumbnail.save(
                            os.path.basename(thumbnail_path),
                            File(thumb_file),
                            save=True
                        )
                    os.remove(thumbnail_path)
                    PostMedia.objects.create(post=post, media=video, media_type='video')

                # If images are uploaded, save all images
                # if images:
                #     for image in images:
                #         PostMedia.objects.create(post=post, media=image, media_type='image')

                # If a video is uploaded, save the single video
                print("Code reached after video creation")
                # print(video)
                # if video:
                #     PostMedia.objects.create(post=post, media=video, media_type='video')

                return Response({
                    "status": "success",
                    "message": "Post created successfully",
                    # "data": PostSerializer(post).data
                }, status=status.HTTP_201_CREATED)
            except Exception as e:
                print(f"Unexpected error: {e}")
                return Response({
                    "status": "error",
                    "message": "An unexpected error occurred",
                    "details": str(e)
                }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
            
        print("Post serializer errors:", post_serializer.errors)
        return Response({
            "status": "error",
            "message": "Failed to create post",
            "errors": post_serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def list_images(self, request, user_id=None):
        try:
            user = get_object_or_404(User, pk=user_id)
            # Filter PostMedia objects with media_type 'image' for the specific user
            # images = PostMedia.objects.filter(post__user=user, media_type='image')
            posts = Post.objects.filter(media_files__media_type='image', user=user, is_deleted=False).distinct().order_by('-created_at')

            if not posts.exists():
                return Response({
                    "status": "error",
                    "message": "No images found for this user."
                }, status=status.HTTP_404_NOT_FOUND)
            
            serializer = PostSerializer(posts, many=True)
            for i in range(len(serializer.data)):
                ############ For like counts ####################
                post_id= posts[i]
                total_likes = Like.objects.filter(post_id=post_id).count()
                serializer.data[i]["like_count"] = total_likes
                user_instance = posts[i].user
                created_on = posts[i].created_at
                current_time = now()
                time_difference = current_time - created_on
                profile_instance = user_instance.profile

                serializer.data[i]["profile_image"] = profile_instance.profile_image.url if profile_instance.profile_image else None

                full_name = user_instance.full_name
                serializer.data[i]["full_name"] = full_name
                post_id = posts[i].id
                follow_list = Follow.objects.filter(follower_id = user.id,following_id= posts[i].user_id )

                if follow_list:
                    serializer.data[i]["follow_status"]=1
                else:
                    serializer.data[i]["follow_status"]=0           
                
                ########  for get total likes #################
                
                total_likes = Like.objects.filter(post_id=post_id).count()
                serializer.data[i]["total_likes"] = total_likes

                ######## For check liked status ###############

                current_user_like = Like.objects.filter(post_id=post_id, user_id = user.id).count()
                serializer.data[i]["liked_status"] = current_user_like

                media_files = serializer.data[i].get("media_files", [])
                if media_files:
                    media_type = media_files[0].get("media_type", "unknown") if isinstance(media_files, list) else "unknown"
                else:
                    media_type = None
                
                # follow_status = Post.objects.filter(post_id= post_id)

                serializer.data[i]["media_type"] = media_type

                if time_difference.total_seconds() <= 23 * 3600:
                    hours, remainder = divmod(time_difference.total_seconds(), 3600)
                    minutes = remainder // 60
                    serializer.data[i]["posted_before"] = f"{int(hours)}h {int(minutes)}m ago" if hours else f"{int(minutes)}m ago"
                else:
                    serializer.data[i]["posted_before"] = created_on.strftime("%Y-%m-%d %H:%M:%S")

                ####################  Comment Counts ####################
                total_comments = Comment.objects.filter(post_id= post_id).count()
                serializer.data[i]["total_comments"] = total_comments

                ###################  Report Status ########################
                report_status = PostReport.objects.filter(post_id=post_id , user_id = user.id ).count()
                serializer.data[i]["report_status"] = report_status

                ################## User_id who posted this ##################

                serializer.data[i]["user_id"]= posts[i].user.id

                ############ Post save status ################
                post_save_status = SavedPost.objects.filter(post_id=post_id, user_id= user.id ).count()
                serializer.data[i]["post_save_status"]= post_save_status

                ############ for comment count ############
            return Response({
                "status": "success",
                "message": "Images retrieved successfully",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        except Exception as e:
            return Response({
                "status": "error",
                "message": "An error occurred while retrieving images.",
                "details": str(e)
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def list_videos(self, request, user_id=None):
        try:
            user = get_object_or_404(User, pk=user_id)
            # Filter PostMedia objects with media_type 'video' for the specific user
            videos = PostMedia.objects.filter(post__user=user, media_type='video', is_deleted=False).order_by('-post__created_at')

            if not videos.exists():
                return Response({
                    "status": "error",
                    "message": "No videos found for this user."
                }, status=status.HTTP_404_NOT_FOUND)

            # Extracting associated Post objects for each media file
            posts = [video.post for video in videos]  # List of posts with videos

            # Serialize the posts (the associated videos are included in the PostSerializer)
            serializer = PostSerializer(posts, many=True)
            for i in range(len(serializer.data)):
                ############ For like counts ####################
                post_id= posts[i]
                total_likes = Like.objects.filter(post_id=post_id).count()
                serializer.data[i]["like_count"] = total_likes
                user_instance = posts[i].user
                created_on = posts[i].created_at
                current_time = now()
                time_difference = current_time - created_on
                profile_instance = user_instance.profile

                serializer.data[i]["profile_image"] = profile_instance.profile_image.url if profile_instance.profile_image else None

                full_name = user_instance.full_name
                serializer.data[i]["full_name"] = full_name
                post_id = posts[i].id
                follow_list = Follow.objects.filter(follower_id = user.id,following_id= posts[i].user_id )

                if follow_list:
                    serializer.data[i]["follow_status"]=1
                else:
                    serializer.data[i]["follow_status"]=0           
                
                ########  for get total likes #################
                
                total_likes = Like.objects.filter(post_id=post_id).count()
                serializer.data[i]["total_likes"] = total_likes

                ######## For check liked status ###############

                current_user_like = Like.objects.filter(post_id=post_id, user_id = user.id).count()
                serializer.data[i]["liked_status"] = current_user_like

                media_files = serializer.data[i].get("media_files", [])
                if media_files:
                    media_type = media_files[0].get("media_type", "unknown") if isinstance(media_files, list) else "unknown"
                else:
                    media_type = None
                
                # follow_status = Post.objects.filter(post_id= post_id)

                serializer.data[i]["media_type"] = media_type

                if time_difference.total_seconds() <= 23 * 3600:
                    hours, remainder = divmod(time_difference.total_seconds(), 3600)
                    minutes = remainder // 60
                    serializer.data[i]["posted_before"] = f"{int(hours)}h {int(minutes)}m ago" if hours else f"{int(minutes)}m ago"
                else:
                    serializer.data[i]["posted_before"] = created_on.strftime("%Y-%m-%d %H:%M:%S")

                ####################  Comment Counts ####################
                total_comments = Comment.objects.filter(post_id= post_id).count()
                serializer.data[i]["total_comments"] = total_comments

                ###################  Report Status ########################
                report_status = PostReport.objects.filter(post_id=post_id , user_id = user.id ).count()
                serializer.data[i]["report_status"] = report_status

                ################## User_id who posted this ##################

                serializer.data[i]["user_id"]= posts[i].user.id

                ############ Post save status ################
                post_save_status = SavedPost.objects.filter(post_id=post_id, user_id= user.id ).count()
                serializer.data[i]["post_save_status"]= post_save_status

            return Response({
                "status": "success",
                "message": "Videos retrieved successfully",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        except Exception as e:
            return Response({
                "status": "error",
                "message": "An error occurred while retrieving videos.",
                "details": str(e)
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
            
    def delete_post(self, request, pk=None):
        post = get_object_or_404(Post, pk=pk)
        
        if post.user != request.user:
            return Response({
                "status": "error",
                "message": "You do not have permission to delete this post."
            }, status=status.HTTP_403_FORBIDDEN)
        
        post.delete()
        return Response({
            "status": "success",
            "message": "Post deleted successfully"
        }, status=status.HTTP_200_OK)


    def like(self, request, pk=None):
        post = get_object_or_404(Post.objects.filter(is_deleted=False), pk=pk)
        like, created = Like.objects.get_or_create(user=request.user, post=post)
        if not created:
            return Response({
                "status": "error",
                "message": "Already liked",
                "total_likes": Like.objects.filter(post=post).count()
            }, status=status.HTTP_400_BAD_REQUEST)
        
        Notification.objects.create(
        user=post.user,  # Notify the owner of the post
        action_type="liked",
        message=f"{request.user.full_name} liked your post."
    )
        if post.user.privacy_settings.enable_notifications:
            response = send_push_notification(
                    fcm_token=post.user.fcm_token,
                    title="liked",
                    body=f"{request.user.full_name} liked your post."
                )
        else:
            response = None 
        total_likes = Like.objects.filter(post=post, is_deleted=False).count()
        return Response({
            "status": "success",
            "message": "Post liked",
            "total_likes": total_likes
        }, status=status.HTTP_201_CREATED)

    def unlike(self, request, pk=None):
        post = get_object_or_404(Post.objects.filter(is_deleted=False), pk=pk)
        Like.objects.filter(user=request.user, post=post).delete()
        total_likes = Like.objects.filter(post=post, is_deleted=False).count()
        return Response({
            "status": "success",
            "message": "Post unliked",
            "total_likes": total_likes
        }, status=status.HTTP_200_OK)

    def comment(self, request, pk=None):
        post = get_object_or_404(Post.objects.filter(is_deleted=False), pk=pk)
        data = request.data.copy()
        data['post'] = pk  
        serializer = CommentSerializer(data=data, context={'request': request})
               
        if serializer.is_valid():
            comment = serializer.save(user=request.user, post=Post.objects.get(id=pk))
            total_comments = Comment.objects.filter(post_id=pk, is_deleted=False).count()

            notification_message = f"{request.user.full_name} commented on your post"
            Notification.objects.create(
                user=post.user,
                action_type="commented",
                message=notification_message
            )
            if post.user.privacy_settings.enable_notifications:
                response = send_push_notification(
                    fcm_token=post.user.fcm_token,
                    title="commented",
                    body=f"{request.user.full_name} commented on your post."
                )
            else:
                response = None 
            print(response)
            return Response({
                "status": "success",
                "message": "Comment added successfully",
                "data": CommentSerializer(comment, context={'request': request}).data,
                "total_comments": total_comments
            }, status=status.HTTP_201_CREATED)    
        return Response({
            "status": "error",
            "message": "Failed to add comment",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def delete_comment(self, request, pk=None, comment_id=None):
        comment = get_object_or_404(Comment, pk=comment_id, post__id=pk, user=request.user)
        comment.delete()
        total_comments = Comment.objects.filter(post_id=pk).count()
        return Response({
            "status": "success",
            "message": "Comment deleted",
            "total_comments": total_comments
        }, status=status.HTTP_200_OK)

    def list_comments(self, request, pk=None):
        post = get_object_or_404(Post, pk=pk)
        comments = post.comments.filter(is_deleted=False)
        serializer = CommentSerializer(comments, many=True,context={"request": request})
        serialized_data = serializer.data
        current_user = request.user
        for i, reply in enumerate(comments):
            like_status = reply.likes.filter(id=current_user.id).exists()
            is_commented_by_user = reply.user == current_user
            serialized_data[i]["comment_like_status"] = True if like_status else False
            serialized_data[i]["comment_status"] = True if is_commented_by_user else False
        return Response({
            "status": "success",
            "message": "Comments retrieved successfully",
            "data": serializer.data
        }, status=status.HTTP_200_OK)
    
    def reply_comment(self, request, pk=None, comment_id=None):
        parent_comment = Comment.objects.get(id=comment_id, is_deleted=False)
        print(parent_comment.parent)

        if parent_comment.parent is not None:
            return Response({
                "status": "error",
                "message": "Replies to replied comments are not allowed."
            }, status=status.HTTP_400_BAD_REQUEST)
        if parent_comment.user.id == request.user.id:
            return Response({
                "status": "error",
                "message": "You can't reply to your own comment."
            }, status=status.HTTP_400_BAD_REQUEST)
        # # if user 
        
        data = request.data.copy()
        data['post'] = pk
        data['parent'] = comment_id
        serializer = CommentSerializer(data=data, context={'request': request})
        # comment_like_count = Comment.objects.filter()



        if serializer.is_valid():
            reply = serializer.save(user=request.user)
            total_replies = Comment.objects.filter(parent_id=comment_id, is_deleted=False).count()
            return Response({
                "status": "success",
                "message": "Reply added successfully",
                "data": CommentSerializer(reply, context={'request': request}).data,
                "total_replies": total_replies
            }, status=status.HTTP_201_CREATED)
        return Response({
            "status": "error",
            "message": "Failed to add reply",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)
    
    def like_comment(self, request, pk=None, comment_id=None):
        comment = get_object_or_404(Comment, pk=comment_id, post__id=pk, is_deleted=False)
        user = request.user

        if user in comment.likes.all():
            comment.likes.remove(user)
            message = "Comment unliked"
        else:
            comment.likes.add(user)
            message = "Comment liked"

        return Response({
            "status": "success",
            "message": message,
            "total_likes": comment.likes.count()
        }, status=status.HTTP_200_OK)
    
    def list_replies(self, request, pk=None, comment_id=None):
        comment = get_object_or_404(Comment, pk=comment_id, post__id=pk, is_deleted=False)
        replies = comment.replies.filter(is_deleted=False)
        serializer = CommentSerializer(replies, many=True)
        serialized_data = serializer.data
        current_user = request.user
        for i, reply in enumerate(replies):
            like_status = reply.likes.filter(id=current_user.id).exists()
            serialized_data[i]["replied_comment_like_status"] = 1 if like_status else 0

        return Response({
            "status": "success",
            "message": "Replies retrieved successfully",
            "data": serializer.data
        }, status=status.HTTP_200_OK)



    def list_likes(self, request, pk=None):
        post = get_object_or_404(Post, pk=pk)
        likes = post.likes.all()
        serializer = LikeSerializer(likes, many=True)
        return Response({
            "status": "success",
            "message": "Likes retrieved successfully",
            "user_id":pk,
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def retrieve(self, request, pk=None):
        """
        View to retrieve details of a single post.
        """
        post = get_object_or_404(Post, pk=pk)
        serializer = PostSerializer(post)
        return Response({
            "status": "success",
            "message": "Post details retrieved successfully",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def share(self, request, pk=None):
        """
        View to generate a shareable link for a specific post.
        """
        post = get_object_or_404(Post, pk=pk)
        username = post.user.username  # Assuming Post model has a user relationship
        # Generate the link to the post detail view
        share_link = f"{request.build_absolute_uri(reverse('post-detail', args=[pk]))}?user={username}"
        return Response({
            "status": "success",
            "message": "Share link generated",
            "share_link": share_link
        }, status=status.HTTP_200_OK)
        
class HeadshotViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]

    def create(self, request):
        banner = request.FILES.get('banner')
        if banner and not banner.content_type.startswith('image/'):
            return Response({
                "status": "error",
                "message": "Invalid file type. Only images are allowed for banners."
            }, status=status.HTTP_400_BAD_REQUEST)

        serializer = HeadshotSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            headshot = serializer.save(user=request.user)
            return Response({
                "status": "success",
                "message": "Headshot created successfully.",
                "data": HeadshotSerializer(headshot, context={'request': request}).data
            }, status=status.HTTP_201_CREATED)

        return Response({
            "status": "error",
            "message": "Failed to create headshot.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def retrieve(self, request, pk=None):
        headshot = get_object_or_404(Headshot, pk=pk)
        serializer = HeadshotSerializer(headshot, context={'request': request})
        return Response({
            "status": "success",
            "message": "Headshot retrieved successfully.",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def list_user_headshots(self, request, user_id=None):
        headshots = Headshot.objects.filter(user__id=user_id)
        if not headshots.exists():
            return Response({
                "status": "error",
                "message": "No headshots found for this user."
            }, status=status.HTTP_404_NOT_FOUND)

        serializer = HeadshotSerializer(headshots, many=True, context={'request': request})
        return Response({
            "status": "success",
            "message": "User headshots retrieved successfully.",
            "data": serializer.data
        }, status=status.HTTP_200_OK)
    
class ListPostFeeds(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        user = request.user
        friends = Follow.objects.filter(
            Q(follower=user) | Q(following=user)
        ).values_list('follower_id', 'following_id')
        friend_ids = set([id for pair in friends for id in pair])
        friend_ids.add(user.id)
        posts = Post.objects.filter(user_id__in=friend_ids, is_deleted=False).order_by('-created_at')
        projects = Project.objects.filter(user_id__in=friend_ids).order_by('-created_at')

        serializer = PostSerializer(posts, many=True)
        project_serializer = ProjectSerializer(projects, many=True, context={'request': request})
        
        print(user.id)
        # print(follow_status)

        for i in range(len(serializer.data)):
            user_instance = posts[i].user
            created_on = posts[i].created_at
            current_time = now()
            time_difference = current_time - created_on
            profile_instance = user_instance.profile

            serializer.data[i]["profile_image"] = profile_instance.profile_image.url if profile_instance.profile_image else None

            full_name = user_instance.full_name
            serializer.data[i]["full_name"] = full_name
            post_id = posts[i].id
            # user_id = posts[i].user_id

            follow_list = Follow.objects.filter(follower_id = user.id,following_id= posts[i].user_id )

            if follow_list:
                serializer.data[i]["follow_status"]=1
            else:
                serializer.data[i]["follow_status"]=0           
            
            ########  for get total likes #################
            
            total_likes = Like.objects.filter(post_id=post_id).count()
            serializer.data[i]["total_likes"] = total_likes

            ######## For check liked status ###############

            current_user_like = Like.objects.filter(post_id=post_id, user_id = user.id).count()
            serializer.data[i]["liked_status"] = current_user_like

            media_files = serializer.data[i].get("media_files", [])
            if media_files:
                media_type = media_files[0].get("media_type", "unknown") if isinstance(media_files, list) else "unknown"
            else:
                media_type = None
            
            # follow_status = Post.objects.filter(post_id= post_id)

            serializer.data[i]["media_type"] = media_type

            if time_difference.total_seconds() <= 23 * 3600:
                hours, remainder = divmod(time_difference.total_seconds(), 3600)
                minutes = remainder // 60
                serializer.data[i]["posted_before"] = f"{int(hours)}h {int(minutes)}m ago" if hours else f"{int(minutes)}m ago"
            else:
                serializer.data[i]["posted_before"] = created_on.strftime("%Y-%m-%d %H:%M:%S")

            ####################  Comment Counts ####################
            total_comments = Comment.objects.filter(post_id= post_id).count()
            serializer.data[i]["total_comments"] = total_comments

            ###################  Report Status ########################
            report_status = PostReport.objects.filter(post_id=post_id , user_id = user.id ).count()
            serializer.data[i]["report_status"] = report_status

            ################## User_id who posted this ##################

            serializer.data[i]["user_id"]= posts[i].user.id

            ############ Post save status ################
            post_save_status = SavedPost.objects.filter(post_id=post_id, user_id= user.id ).count()
            serializer.data[i]["post_save_status"]= post_save_status

        combined_data = serializer.data + project_serializer.data

        # Add type field for differentiation
        for item in serializer.data:
            item['type'] = 'post'
        for item in project_serializer.data:
            item['type'] = 'project'

        # Sort combined data by created_at
        sorted_data = sorted(combined_data, key=itemgetter('created_at'), reverse=True)

        return Response({"data": sorted_data}, status=200)

        
        # return Response({"data": serializer.data}, status=200)
    
class ReportReasonListAPIView(APIView):

    def get(self, request, *args, **kwargs):
        reasons = ReportReason.objects.all()
        serializer = ReportReasonSerializer(reasons, many=True)
        # return Response(serializer.data, status=status.HTTP_200_OK)
        return Response({"status": "success", "data": serializer.data})
    
class ReportPostAPIView(APIView):

    def post(self, request, *args, **kwargs):
        serializer = PostReportSerializer(data=request.data)
        if serializer.is_valid():
            post = get_object_or_404(Post, id=serializer.validated_data['post'].id)
            reason = get_object_or_404(ReportReason, id=serializer.validated_data['reason'].id)

            # if Post.objects.filter(post=post, user=request.user).exists():
            #     return Response({"detail": "You can not report on your own profile"}, status=status.HTTP_400_BAD_REQUEST)

            # Check if user has already reported this post
            if PostReport.objects.filter(post=post, user=request.user).exists():
                return Response({"detail": "You have already reported this post."}, status=status.HTTP_400_BAD_REQUEST)

            # Save the report
            serializer.save(user=request.user)
            return Response({"detail": "Report submitted successfully."}, status=status.HTTP_201_CREATED)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
class PostReportStatusAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, post_id, *args, **kwargs):
        user = request.user

        try:
            # Check if the post is reported by the logged-in user
            is_reported = PostReport.objects.filter(post_id=post_id, user=user).exists()

            if is_reported:
                report = PostReport.objects.get(post_id=post_id, user=user)
                return Response({
                    "status": "success",
                    "post_id": post_id,
                    "reported": True,
                    "reported_at": report.reported_at,
                    "reason_id": report.reason_id
                }, status=200)
            else:
                return Response({
                    "status": "success",
                    "post_id": post_id,
                    "reported": False
                }, status=200)

        except Exception as e:
            return Response({
                "error": "failed",
                "message": f"An unexpected error occurred: {str(e)}"
            }, status=500)